from dataclasses import dataclass
from typing import List, Optional

from dataclasses_json import dataclass_json


@dataclass_json
@dataclass(frozen=True)
class Node:
    id: int
    int_ip: str
    node_group: str
    node_type: str
    url: str
    display_name: Optional[str] = None


Nodes = List[Node]



class JelasticClientException(Exception):
    def __init__(self, message: str, response=None):
        super().__init__(message)
        if response is None:
            response = {}
        self.response = response



import json
from typing import Optional

import requests  # type: ignore

from .core import ApiClient, BaseClient, JelasticClientException, who_am_i


class JpsClient(BaseClient):
    jelastic_group = "marketplace"

    jelastic_class = "jps"

    def __init__(self, api_client: ApiClient):
        super().__init__(api_client)

    def install_from_file(
        self,
        filename: str,
        env_name: Optional[str] = None,
        settings: Optional[dict] = None,
        region: Optional[str] = None,
    ) -> str:
        try:
            file = open(filename, "r")
        except OSError:
            raise JelasticClientException(f"Unable to open file {filename}")

        with file:
            manifest_content = file.read()
            return self.install(manifest_content, env_name, settings, region=region)

    def install_from_url(
        self,
        url: str,
        env_name: Optional[str] = None,
        settings: Optional[dict] = None,
        region: Optional[str] = None,
    ) -> str:
        response = requests.get(url)
        if response.status_code != 200:
            raise JelasticClientException(f"Url not found: {url}")
        manifest_content = response.text
        return self.install(manifest_content, env_name, settings, region=region)

    def install(
        self,
        manifest_content: str,
        env_name: Optional[str] = None,
        settings: Optional[dict] = None,
        region: Optional[str] = None,
    ) -> str:
        response = self._execute(
            who_am_i(),
            jps=manifest_content,
            envName=env_name,
            skipNodeEmails=True,
            settings=json.dumps(settings),
            region=region,
        )

        return response["successText"]

    def get_engine_version(self) -> str:
        response = self._execute(who_am_i())

        return response["version"]



from dataclasses import dataclass
from typing import List, Optional

from dataclasses_json import dataclass_json


@dataclass_json
@dataclass(frozen=True)
class DockerSettings:
    image: str
    nodeGroup: Optional[str] = None


@dataclass_json
@dataclass(frozen=True)
class NodeSettings:
    docker: Optional[DockerSettings] = None
    count: int = 1
    displayName: Optional[str] = None
    extip: bool = False
    fixedCloudlets: Optional[int] = None
    flexibleCloudlets: Optional[int] = None
    nodeType: Optional[str] = None


MultipleNodeSettings = List[NodeSettings]



import json
import random
import string
from typing import List, Optional

from .core import ApiClient, BaseClient, JelasticClientException, who_am_i
from .env_info import EnvInfo
from .env_settings import EnvSettings
from .env_status import EnvStatus
from .node_settings import MultipleNodeSettings, NodeSettings


class ControlClient(BaseClient):
    jelastic_group = "environment"

    jelastic_class = "control"

    def __init__(self, api_client: ApiClient):
        super().__init__(api_client)

    def generate_random_env_name(self):
        env_name = self._create_random_env_name()
        while self.get_env_info(env_name).exists():
            env_name = self._create_random_env_name()
        return env_name

    def create_environment(
        self, env: EnvSettings, nodes: MultipleNodeSettings
    ) -> EnvInfo:
        env_json = env.to_json()  # type: ignore
        nodes_json = NodeSettings.schema().dumps(nodes, many=True)  # type: ignore
        response = self._execute(who_am_i(), env=env_json, nodes=nodes_json)
        return EnvInfo(response["response"])

    def delete_env(self, env_name: str) -> None:
        self._execute(who_am_i(), envName=env_name)

    def clone_env(self, src_env_name: str, dst_env_name: str) -> EnvInfo:
        response = self._execute(
            who_am_i(), srcEnvName=src_env_name, dstEnvName=dst_env_name
        )
        return EnvInfo(response)

    def start_env(self, env_name: str) -> None:
        self._execute(who_am_i(), envName=env_name)

    def stop_env(self, env_name: str) -> None:
        self._execute(who_am_i(), envName=env_name)

    def get_env_info(self, env_name: str) -> EnvInfo:
        try:
            response = self._execute(who_am_i(), envName=env_name)
        except JelasticClientException as e:
            response = {
                "result": e.response["result"],
                "env": {"status": EnvStatus.NotExists},
            }
        return EnvInfo(response)

    def get_container_env_vars_by_group(self, env_name: str, node_group: str) -> dict:
        response = self._execute(who_am_i(), envName=env_name, nodeGroup=node_group)
        return response["object"]

    def get_container_env_vars(self, env_name: str, node_id: int):
        response = self._execute(who_am_i(), envName=env_name, nodeId=node_id)
        return response["object"]

    def remove_container_env_vars(
        self,
        env_name: str,
        vars: List[str],
        node_group: Optional[str] = None,
        node_id: Optional[str] = None,
    ):
        self._execute(
            who_am_i(),
            envName=env_name,
            nodeGroup=node_group,
            nodeId=node_id,
            vars=json.dumps(vars),
        )

    def add_container_env_vars(
        self,
        env_name: str,
        vars: dict,
        node_group: Optional[str] = None,
        node_id: Optional[int] = None,
    ):
        self._execute(
            who_am_i(),
            envName=env_name,
            nodeGroup=node_group,
            nodeId=node_id,
            vars=json.dumps(vars),
        )

    def set_container_env_vars(self, env_name: str, node_id: int, vars: dict):
        self._execute(
            who_am_i(), envName=env_name, nodeId=node_id, vars=json.dumps(vars)
        )

    def set_container_env_vars_by_group(
        self, env_name: str, node_group: str, vars: dict
    ):
        self._execute(
            who_am_i(), envName=env_name, nodeGroup=node_group, data=json.dumps(vars)
        )

    @staticmethod
    def _create_random_env_name():
        env_id = "".join(random.choice(string.digits) for _ in range(7))
        return f"env-{env_id}"



from typing import Optional

from jelastic_client.core import ApiClient, BaseClient, who_am_i


class FileClient(BaseClient):
    jelastic_group = "environment"

    jelastic_class = "file"

    def __init__(self, api_client: ApiClient):
        super().__init__(api_client)

    def read(
        self,
        env_name: str,
        path: str,
        node_type: Optional[str] = None,
        node_group: Optional[str] = None,
        node_id: Optional[str] = None,
    ) -> str:
        response = self._execute(
            who_am_i(),
            envName=env_name,
            path=path,
            nodeType=node_type,
            nodeGroup=node_group,
            nodeid=node_id,
        )
        return response["body"]



from __future__ import annotations

from dataclasses import dataclass
from typing import Optional

from dataclasses_json import dataclass_json


@dataclass_json
@dataclass(frozen=True)
class EnvSettings:
    displayName: Optional[str] = None
    engine: Optional[str] = None
    ishaenabled: bool = False
    region: Optional[str] = None
    shortdomain: Optional[str] = None
    sslstate: bool = False



from .account_client import AccountClient
from .control_client import ControlClient
from .core import JelasticClientException
from .env_info import EnvInfo
from .env_settings import EnvSettings
from .env_status import EnvStatus
from .factory import JelasticClientFactory
from .file_client import FileClient
from .jps_client import JpsClient
from .node import Node, Nodes
from .node_settings import DockerSettings, NodeSettings



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jelastic_client', 'jelastic_client.core']

package_data = \
{'': ['*']}

install_requires = \
['dataclasses-json==0.5.7', 'httpx[http2]>=0.18', 'requests==2.28.1']

setup_kwargs = {
    'name': 'jelastic-client',
    'version': '1.3.29',
    'description': 'A client library for Jelastic',
    'long_description': '# jelastic-client\n\nA Jelastic API python library.\n\n# Installation\n\n```bash\npip3 install jelastic-client\n```\n\n# Usage\n\nAt the root of this repository, you can run\n\n```python\nimport jelastic_client\n\napi_url = "https://[hoster-api-host]/1.0/"\napi_token = "your-private-access-token"\n\nfactory = jelastic_client.JelasticClientFactory(api_url, api_token)\njps_client = factory.create_jps_client()\nenv_name = "my-jelastic-client-test"\njps_client.install_from_file("./test/data/valid_manifest.jps", env_name)\ncontrol_client = factory.create_control_client()\nenv_info = control_client.get_env_info(env_name)\nassert env_info.is_running() is True\n```\n',
    'author': 'softozor',
    'author_email': 'softozor@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
